searchData={"items":[{"type":"module","title":"EctoShorts","doc":"Ecto Shorts is a library created to help shorten ecto interactions\nand remove most of the related code.\n\n\n\nThere are 2 main modules `EctoShorts.Actions` and `EctoShorts.CommonChanges`","ref":"EctoShorts.html"},{"type":"module","title":"Actions - EctoShorts","doc":"`EctoShorts.Actions` allows for filters to be constructed from data such as\n\n```elixir\nActions.all(User, %{\n  favorite_food: \"curry\",\n  age: %{gte: 18, lte: 50},\n  name: %{ilike: \"steven\"},\n  preload: [:address],\n  last: 5\n})\n```\n\nwhich the equivelent would be\n\n```elixir\nRepo.all from u in User,\n  preload: [:address],\n  limit: 5,\n  where: u.favorite_food == \"curry\" and\n         u.age >= 18 and u.age <= 50 and\n         ilike(u.name, \"%steven%\")\n```\nAll actions can accept an optional argument of a keyword list that can be used to configure which Repo the Action should use.","ref":"EctoShorts.html#module-actions"},{"type":"module","title":"Options - EctoShorts","doc":"* `:repo` - A module that uses the Ecto.Repo Module.\n  * `:replica` - If you don't want to perform any reads against your Primary, you can specify a replica to read from.\n\n\n\nSee `EctoShorts.CommonFilters` for more info on filters","ref":"EctoShorts.html#module-options"},{"type":"module","title":"CommonChanges - EctoShorts","doc":"`EctoShorts.CommonChanges` is responsible for updating relations on schemas it allows\nyou to update associations or create using changesets and handles choosing between\n`put_assoc` and `cast_assoc`\n\n```elixir\nActions.update(User, 1, %{friends: [%{id: 5, name: \"Billy\"}]})\nActions.update(User, 1, %{friends: [%{id: 5, name: \"Bob\"}]})\n```\n\nAnother feature is you can update many to many relationship members using a id list of members\n\n###### How to make user 1 friends with users 5, 6, 7\n```elixir\nActions.update(User, 1, %{friends: [%{id: 5}, %{id: 6}, %{id: 7}]})\n```","ref":"EctoShorts.html#module-commonchanges"},{"type":"module","title":"Config - EctoShorts","doc":"You can configure both the Repo and Error module via\n```elixir\nconfig :ecto_shorts,\n  repo: MyRepo,\n  error_module: EctoShorts.Actions.Error\n```","ref":"EctoShorts.html#module-config"},{"type":"module","title":"EctoShorts.Actions","doc":"Actions for CRUD in ecto, these can be used by all schemas/queries\n\nGenerally we can define our contexts to be very reusable by creating\nthem to look something like this:\n\n\n    defmodule MyApp.Accounts do\n      alias EctoShorts.Actions\n      alias MyApp.Accounts.User\n\n      def all_users(params), do: Actions.all(User, params)\n      def find_user(params), do: Actions.find(User, params)\n    end\n\nWe're then able to use this context with all filters that are\nsupported by `EctoShorts.CommonFilters` without having to create new queries\n\n\n    def do_something do\n      MyApp.Accounts.all_user(%{\n        first_name: %{ilike: \"john\"},\n        age: %{gte: 18},\n        priority_level: 5,\n        address: %{country: \"Canada\"}\n      })\n    end\n\n\nYou can read more on reusable ecto code [here](https://learn-elixir.dev/blogs/creating-reusable-ecto-code)","ref":"EctoShorts.Actions.html"},{"type":"module","title":"Supporting multiple Repos - EctoShorts.Actions","doc":"To support multiple repos, what we can do is pass arguments to the last parameter\nof most `EctoShorts.Actions` calls\n\n#### Example\n\n    defmodule MyApp.Accounts do\n      alias EctoShorts.Actions\n      alias MyApp.Accounts.User\n\n      @repo [repo: MyApp.Repo.Replica1]\n\n      def all_users(params), do: Actions.all(User, params, @repo)\n      def find_user(params), do: Actions.find(User, params, @repo)\n    end","ref":"EctoShorts.Actions.html#module-supporting-multiple-repos"},{"type":"function","title":"EctoShorts.Actions.aggregate/5","doc":"","ref":"EctoShorts.Actions.html#aggregate/5"},{"type":"function","title":"EctoShorts.Actions.all/1","doc":"Gets a collection of schemas from the database","ref":"EctoShorts.Actions.html#all/1"},{"type":"function","title":"Examples - EctoShorts.Actions.all/1","doc":"iex> EctoSchemas.Actions.all(EctoSchemas.Accounts.User)\n    []","ref":"EctoShorts.Actions.html#all/1-examples"},{"type":"function","title":"EctoShorts.Actions.all/2","doc":"Gets a collection of schemas from the database but allows for a filter or\nan options list.","ref":"EctoShorts.Actions.html#all/2"},{"type":"function","title":"Options - EctoShorts.Actions.all/2","doc":"* `:repo` - A module that uses the Ecto.Repo Module.","ref":"EctoShorts.Actions.html#all/2-options"},{"type":"function","title":"Examples - EctoShorts.Actions.all/2","doc":"iex> Enum.each(1..4, fn _ -> create_user() end)\n    iex> length(EctoSchemas.Actions.all(EctoSchemas.Accounts.User, first: 3)) === 3\n    true\n\n    iex> Enum.each(1..4, fn _ -> create_user() end)\n    iex> length(EctoSchemas.Actions.all(EctoSchemas.Accounts.User, repo: MyApp.MyRepoModule.Repo)) === 3\n    true","ref":"EctoShorts.Actions.html#all/2-examples"},{"type":"function","title":"EctoShorts.Actions.all/3","doc":"Similar to `all/2` but can also accept a keyword options list.","ref":"EctoShorts.Actions.html#all/3"},{"type":"function","title":"Options - EctoShorts.Actions.all/3","doc":"* `:repo` - A module that uses the Ecto.Repo Module.","ref":"EctoShorts.Actions.html#all/3-options"},{"type":"function","title":"Examples - EctoShorts.Actions.all/3","doc":"iex> Enum.each(1..4, fn _ -> create_user() end)\n  iex> length(EctoSchemas.Actions.all(EctoSchemas.Accounts.User, first: 3, repo: MyApp.MyRepoModule.Repo)) === 3\n  true","ref":"EctoShorts.Actions.html#all/3-examples"},{"type":"function","title":"EctoShorts.Actions.create/3","doc":"Creates a schema with given params. Can also accept a keyword options list.","ref":"EctoShorts.Actions.html#create/3"},{"type":"function","title":"Options - EctoShorts.Actions.create/3","doc":"* `:repo` - A module that uses the Ecto.Repo Module.","ref":"EctoShorts.Actions.html#create/3-options"},{"type":"function","title":"Examples - EctoShorts.Actions.create/3","doc":"iex> {:ok, schema} = EctoSchemas.Actions.create(EctoSchemas.Accounts.User, user_params(first_name: \"TEST\"))\n    iex> schema.first_name\n    \"TEST\"\n    iex> {:error, changeset} = EctoSchemas.Actions.create(EctoSchemas.Accounts.User, Map.delete(user_params(), :first_name))\n    iex> \"can't be blank\" in errors_on(changeset).first_name\n    true","ref":"EctoShorts.Actions.html#create/3-examples"},{"type":"function","title":"Examples - EctoShorts.Actions.create/3","doc":"iex> {:ok, schema} = EctoSchemas.Actions.create(EctoSchemas.Accounts.User, user_params(first_name: \"TEST\"), repo: MyApp.MyRepoModule.Repo)\n    iex> schema.first_name\n    true","ref":"EctoShorts.Actions.html#create/3-examples"},{"type":"function","title":"EctoShorts.Actions.delete/1","doc":"Deletes a schema","ref":"EctoShorts.Actions.html#delete/1"},{"type":"function","title":"Examples - EctoShorts.Actions.delete/1","doc":"iex> user = create_user()\n    iex> {:ok, schema} = EctoSchemas.Actions.delete(user)\n    iex> schema.first_name === user.first_name\n    true","ref":"EctoShorts.Actions.html#delete/1-examples"},{"type":"function","title":"EctoShorts.Actions.delete/2","doc":"Similar to `delete/1` but can also accept a keyword options list.","ref":"EctoShorts.Actions.html#delete/2"},{"type":"function","title":"Options - EctoShorts.Actions.delete/2","doc":"* `:repo` - A module that uses the Ecto.Repo Module.","ref":"EctoShorts.Actions.html#delete/2-options"},{"type":"function","title":"Examples - EctoShorts.Actions.delete/2","doc":"iex> user = create_user()\n    iex> {:ok, schema} = EctoSchemas.Actions.delete(user, repo: MyApp.MyRepoModule.Repo)\n    iex> schema.first_name === user.first_name\n    true","ref":"EctoShorts.Actions.html#delete/2-examples"},{"type":"function","title":"EctoShorts.Actions.delete/3","doc":"Deletes a schema. Can also accept a keyword options list.","ref":"EctoShorts.Actions.html#delete/3"},{"type":"function","title":"Options - EctoShorts.Actions.delete/3","doc":"* `:repo` - A module that uses the Ecto.Repo Module.","ref":"EctoShorts.Actions.html#delete/3-options"},{"type":"function","title":"Examples - EctoShorts.Actions.delete/3","doc":"iex> user = create_user()\n    iex> {:ok, schema} = EctoSchemas.Actions.delete(EctoSchemas.Accounts.User, user.id)\n    iex> schema.first_name === user.first_name\n    true\n\n    iex> user = create_user()\n    iex> {:ok, schema} = EctoSchemas.Actions.delete(EctoSchemas.Accounts.User, user.id)\n    iex> schema.first_name === user.first_name\n    true","ref":"EctoShorts.Actions.html#delete/3-examples"},{"type":"function","title":"EctoShorts.Actions.find/3","doc":"Finds a schema with matching params. Can also accept a keyword options list.","ref":"EctoShorts.Actions.html#find/3"},{"type":"function","title":"Options - EctoShorts.Actions.find/3","doc":"* `:repo` - A module that uses the Ecto.Repo Module.","ref":"EctoShorts.Actions.html#find/3-options"},{"type":"function","title":"Examples - EctoShorts.Actions.find/3","doc":"iex> user = create_user()\n    iex> {:ok, schema} = EctoSchemas.Actions.find(EctoSchemas.Accounts.User, first_name: user.first_name)\n    iex> schema.first_name === user.first_name\n    true\n\n    iex> user = create_user()\n    iex> {:ok, schema} = EctoSchemas.Actions.find(EctoSchemas.Accounts.User, first_name: user.first_name, repo: MyApp.MyRepoModule.Repo)\n    iex> schema.first_name === user.first_name\n    true","ref":"EctoShorts.Actions.html#find/3-examples"},{"type":"function","title":"EctoShorts.Actions.find_and_update/4","doc":"Finds a schema by params and updates it or creates with results of\nparams/update_params merged. Can also accept a keyword options list.\n\n***Note: Relational filtering doesn't work on this function***","ref":"EctoShorts.Actions.html#find_and_update/4"},{"type":"function","title":"Options - EctoShorts.Actions.find_and_update/4","doc":"* `:repo` - A module that uses the Ecto.Repo Module.\n  * `:replica` - If you don't want to perform any reads against your Primary, you can specify a replica to read from.","ref":"EctoShorts.Actions.html#find_and_update/4-options"},{"type":"function","title":"Examples - EctoShorts.Actions.find_and_update/4","doc":"iex> {:ok, schema} = EctoSchemas.Actions.find_and_update(EctoSchemas.Accounts.User, %{email: \"some_email\"}, %{name: \"great name\"})\n\n    iex> {:ok, schema} = EctoSchemas.Actions.find_and_update(EctoSchemas.Accounts.User, %{email: \"some_email\"}, %{name: \"great name}, repo: MyApp.MyRepoModule.Repo, replica: MyApp.MyRepoModule.Repo.replica())","ref":"EctoShorts.Actions.html#find_and_update/4-examples"},{"type":"function","title":"EctoShorts.Actions.find_and_upsert/4","doc":"Finds a schema by params and updates it or creates with results of\nparams/update_params merged. Can also accept a keyword options list.\n\n***Note: Relational filtering doesn't work on this function***","ref":"EctoShorts.Actions.html#find_and_upsert/4"},{"type":"function","title":"Options - EctoShorts.Actions.find_and_upsert/4","doc":"* `:repo` - A module that uses the Ecto.Repo Module.\n  * `:replica` - If you don't want to perform any reads against your Primary, you can specify a replica to read from.","ref":"EctoShorts.Actions.html#find_and_upsert/4-options"},{"type":"function","title":"Examples - EctoShorts.Actions.find_and_upsert/4","doc":"iex> {:ok, schema} = EctoSchemas.Actions.find_and_upsert(EctoSchemas.Accounts.User, %{email: \"some_email\"}, %{name: \"great name\"})\n\n    iex> {:ok, schema} = EctoSchemas.Actions.find_and_upsert(EctoSchemas.Accounts.User, %{email: \"some_email\"}, %{name: \"great name}, repo: MyApp.MyRepoModule.Repo, replica: MyApp.MyRepoModule.Repo.replica())","ref":"EctoShorts.Actions.html#find_and_upsert/4-examples"},{"type":"function","title":"EctoShorts.Actions.find_or_create/3","doc":"Finds a schema by params or creates one if it isn't found.\nCan also accept a keyword options list.\n\n***Note: Relational filtering doesn't work on this function***","ref":"EctoShorts.Actions.html#find_or_create/3"},{"type":"function","title":"Options - EctoShorts.Actions.find_or_create/3","doc":"* `:repo` - A module that uses the Ecto.Repo Module.\n  * `:replica` - If you don't want to perform any reads against your Primary, you can specify a replica to read from.","ref":"EctoShorts.Actions.html#find_or_create/3-options"},{"type":"function","title":"Examples - EctoShorts.Actions.find_or_create/3","doc":"iex> {:ok, schema} = EctoSchemas.Actions.find_or_create(EctoSchemas.Accounts.User, %{name: \"great name\"})\n\n    iex> {:ok, schema} = EctoSchemas.Actions.find_or_create(EctoSchemas.Accounts.User, %{name: \"great name\"}, repo: MyApp.MyRepoModule.Repo, replica: MyApp.MyRepoModule.Repo.replica())","ref":"EctoShorts.Actions.html#find_or_create/3-examples"},{"type":"function","title":"EctoShorts.Actions.find_or_create_many/3","doc":"Accepts a list of schemas and attempts to find them in the DB. Any missing Schemas will be created.\nCan also accept a keyword options list.\n\n***Note: Relational filtering doesn't work on this function***","ref":"EctoShorts.Actions.html#find_or_create_many/3"},{"type":"function","title":"Options - EctoShorts.Actions.find_or_create_many/3","doc":"* `:repo` - A module that uses the Ecto.Repo Module.\n  * `:replica` - If you don't want to perform any reads against your Primary, you can specify a replica to read from.","ref":"EctoShorts.Actions.html#find_or_create_many/3-options"},{"type":"function","title":"Examples - EctoShorts.Actions.find_or_create_many/3","doc":"iex> {:ok, records} = EctoSchemas.Actions.find_or_create_many(EctoSchemas.Accounts.User, [%{name: \"foo\"}, %{name: \"bar}])\n  iex> length(records) === 2","ref":"EctoShorts.Actions.html#find_or_create_many/3-examples"},{"type":"function","title":"EctoShorts.Actions.get/3","doc":"Gets a schema from the database","ref":"EctoShorts.Actions.html#get/3"},{"type":"function","title":"Examples - EctoShorts.Actions.get/3","doc":"iex> user = create_user()\n    iex> %{id: id} = EctoSchemas.Actions.get(EctoSchemas.Accounts.User, user.id)\n    iex> id === user.id\n    true\n    iex> EctoSchemas.Actions.get(EctoSchemas.Accounts.User, 2504390) # ID nonexistant\n    nil","ref":"EctoShorts.Actions.html#get/3-examples"},{"type":"function","title":"EctoShorts.Actions.stream/3","doc":"Gets a collection of schemas from the database but allows for a filter","ref":"EctoShorts.Actions.html#stream/3"},{"type":"function","title":"EctoShorts.Actions.update/4","doc":"Updates a schema with given updates. Can also accept a keyword options list.","ref":"EctoShorts.Actions.html#update/4"},{"type":"function","title":"Options - EctoShorts.Actions.update/4","doc":"* `:repo` - A module that uses the Ecto.Repo Module.\n  * `:replica` - If you don't want to perform any reads against your Primary, you can specify a replica to read from.","ref":"EctoShorts.Actions.html#update/4-options"},{"type":"function","title":"Examples - EctoShorts.Actions.update/4","doc":"iex> user = create_user()\n    iex> {:ok, schema} = EctoSchemas.Actions.update(EctoSchemas.Accounts.User, user, first_name: user.first_name)\n    iex> schema.first_name === user.first_name\n    true\n\n    iex> user = create_user()\n    iex> {:ok, schema} = EctoSchemas.Actions.update(EctoSchemas.Accounts.User, 1, first_name: user.first_name, repo: MyApp.MyRepoModule.Repo, replica: MyApp.MyRepoModule.Repo.replica())\n    iex> schema.first_name === user.first_name\n    true","ref":"EctoShorts.Actions.html#update/4-examples"},{"type":"type","title":"EctoShorts.Actions.aggregate_options/0","doc":"","ref":"EctoShorts.Actions.html#t:aggregate_options/0"},{"type":"type","title":"EctoShorts.Actions.filter_params/0","doc":"","ref":"EctoShorts.Actions.html#t:filter_params/0"},{"type":"type","title":"EctoShorts.Actions.opts/0","doc":"","ref":"EctoShorts.Actions.html#t:opts/0"},{"type":"type","title":"EctoShorts.Actions.query/0","doc":"","ref":"EctoShorts.Actions.html#t:query/0"},{"type":"type","title":"EctoShorts.Actions.schema_list/0","doc":"","ref":"EctoShorts.Actions.html#t:schema_list/0"},{"type":"type","title":"EctoShorts.Actions.schema_res/0","doc":"","ref":"EctoShorts.Actions.html#t:schema_res/0"},{"type":"module","title":"EctoShorts.CommonChanges","doc":"`CommonChanges` is a collection of functions to help with managing\nand creating our `&changeset/2` function in our schemas.","ref":"EctoShorts.CommonChanges.html"},{"type":"module","title":"Preloading associations on change - EctoShorts.CommonChanges","doc":"Often times we want to be able to change an association with\n`(put/cast)_assoc`, but we have an awkwardness of having to use\na preload in a spot to do this. We can aleviate that by doing the following:\n\n    defmodule MyApp.Accounts.User do\n      def changeset(changeset, params) do\n        changeset\n          |> cast([:name, :email])\n          |> validate_required([:name, :email])\n          |> EctoShorts.CommonChanges.preload_change_assoc(:address)\n      end\n    end\n\nDoing this allows us to then pass address in via a map, or even using\nthe struct from the database directly to add as a relation","ref":"EctoShorts.CommonChanges.html#module-preloading-associations-on-change"},{"type":"module","title":"Validating relation is passed in somehow - EctoShorts.CommonChanges","doc":"We can validate for a relation being passed in via id or by using our\npreload_change_assoc by doing the following:\n\n    defmodule MyApp.Accounts.User do\n      def changeset(changeset, params) do\n        changeset\n          |> cast([:name, :email, :address_id])\n          |> validate_required([:name, :email])\n          |> EctoShorts.CommonChanges.preload_change_assoc(:address,\n            required_when_missing: :address_id\n          )\n      end\n    end","ref":"EctoShorts.CommonChanges.html#module-validating-relation-is-passed-in-somehow"},{"type":"module","title":"Conditional functions - EctoShorts.CommonChanges","doc":"We can also run functions when something happens by defining conditional functions like so:\n\n    defmodule MyApp.Accounts.User do\n      alias EctoShorts.CommonChanges\n\n      def changeset(changeset, params) do\n        changeset\n          |> cast([:name, :email, :address_id])\n          |> validate_required([:name, :email])\n          |> CommonChanges.put_when(\n            &CommonChanges.changeset_field_nil?(&1, :email),\n            &put_change(&1, :email, \"some_default@gmail.com\")\n          )\n      end\n    end","ref":"EctoShorts.CommonChanges.html#module-conditional-functions"},{"type":"function","title":"EctoShorts.CommonChanges.changeset_field_empty?/2","doc":"Checks if field on changeset is empty list in data or changes","ref":"EctoShorts.CommonChanges.html#changeset_field_empty?/2"},{"type":"function","title":"EctoShorts.CommonChanges.changeset_field_nil?/2","doc":"Checks if field on changeset is nil in data or changes","ref":"EctoShorts.CommonChanges.html#changeset_field_nil?/2"},{"type":"function","title":"EctoShorts.CommonChanges.default_opts/0","doc":"","ref":"EctoShorts.CommonChanges.html#default_opts/0"},{"type":"function","title":"EctoShorts.CommonChanges.preload_change_assoc/2","doc":"","ref":"EctoShorts.CommonChanges.html#preload_change_assoc/2"},{"type":"function","title":"EctoShorts.CommonChanges.preload_change_assoc/3","doc":"This function is the primary use function\nPreloads changeset assoc if change is made and then and put_or_cast's it","ref":"EctoShorts.CommonChanges.html#preload_change_assoc/3"},{"type":"function","title":"Example - EctoShorts.CommonChanges.preload_change_assoc/3","doc":"iex> CommonChanges.preload_change_assoc(changeset, :my_relation)\n  iex> CommonChanges.preload_change_assoc(changeset, :my_relation, repo: MyApp.OtherRepo)\n  iex> CommonChanges.preload_change_assoc(changeset, :my_relation, required: true)\n  iex> CommonChanges.preload_change_assoc(changeset, :my_relation, required_when_missing: :my_relation_id)","ref":"EctoShorts.CommonChanges.html#preload_change_assoc/3-example"},{"type":"function","title":"EctoShorts.CommonChanges.preload_changeset_assoc/3","doc":"Preloads a changesets association","ref":"EctoShorts.CommonChanges.html#preload_changeset_assoc/3"},{"type":"function","title":"EctoShorts.CommonChanges.put_or_cast_assoc/3","doc":"Determines put or cast on association with some special magic\n\nIf you pass a many to many relation only a list of id's it will count that as a `member_update` and remove or add members to the relations list\n\nE.G. User many_to_many Fruit\n\nThis would update the user to have only fruits with id 1 and 3\n```elixir\nCommonChanges.put_or_cast_assoc(change(user, fruits: [%{id: 1}, %{id: 3}]), :fruits)\n```","ref":"EctoShorts.CommonChanges.html#put_or_cast_assoc/3"},{"type":"function","title":"EctoShorts.CommonChanges.put_when/3","doc":"Run's changeset function if when function returns true","ref":"EctoShorts.CommonChanges.html#put_when/3"},{"type":"module","title":"EctoShorts.CommonFilters","doc":"This modules main purpose is to house a collection of common schema filters\nand functionality to be included in params -> filters\n\nCommon filters available include\n\n- `preload` - Preloads fields onto the query results\n- `start_date` - Query for items inserted after this date\n- `end_date` - Query for items inserted before this date\n- `before` - Get items with ID's before this value\n- `after` - Get items with ID's after this value\n- `ids` - Get items with a list of ids\n- `first` - Gets the first n items\n- `last` - Gets the last n items\n- `limit` - Gets the first n items\n- `offset` - Offsets limit by n items\n- `order_by` - orders the results in desc or asc order\n- `search` - ***Warning:*** This requires schemas using this to have a `&by_search(query, val)` function\n\n```elixir\nCommonFilters.convert_params_to_filter(User, %{first: 10})\nCommonFilters.convert_params_to_filter(User, %{ids: [1, 2, 3, 4]})\nCommonFilters.convert_params_to_filter(User, %{order_by: {:desc, :email_updated_at})\n```\n\nYou are also able to filter on any natural field of a model, as well as use\n\n- gte/gt\n- lte/lt\n- like/ilike\n- is_nil/not(is_nil)\n\n```elixir\nCommonFilters.convert_params_to_filter(User, %{name: \"Billy\"})\nCommonFilters.convert_params_to_filter(User, %{name: %{ilike: \"steve\"}})\nCommonFilters.convert_params_to_filter(User, %{name: %{age: %{gte: 18, lte: 30}}})\nCommonFilters.convert_params_to_filter(User, %{name: %{is_banned: %{!=: nil}}})\nCommonFilters.convert_params_to_filter(User, %{name: %{is_banned: %{==: nil}}})\nCommonFilters.convert_params_to_filter(User, %{name: %{balance: %{!=: 0}}})\n```\n\nCommonFilters also supports limited fragment modifiers of natural fields:\n\n- :lower for \"lower(?)\"\n- :upper for \"lower(?)\"\n\n```elixir\nCommonFilters.convert_params_to_filter(User, %{name: {:lower, \"billy\"}})\nCommonFilters.convert_params_to_filter(User, %{name: {:upper, \"BILLY\"}})\nCommonFilters.convert_params_to_filter(User, %{name: %{!=: {:lower, \"billy\"}}})\n```","ref":"EctoShorts.CommonFilters.html"},{"type":"function","title":"EctoShorts.CommonFilters.convert_params_to_filter/2","doc":"Converts filter params into a query","ref":"EctoShorts.CommonFilters.html#convert_params_to_filter/2"},{"type":"function","title":"EctoShorts.CommonFilters.create_schema_filter/2","doc":"","ref":"EctoShorts.CommonFilters.html#create_schema_filter/2"},{"type":"module","title":"EctoShorts.SchemaHelpers","doc":"Module that has helpers that are globably useful on ecto schemas","ref":"EctoShorts.SchemaHelpers.html"},{"type":"function","title":"EctoShorts.SchemaHelpers.all_created?/1","doc":"Determine if all items in list has been created or not","ref":"EctoShorts.SchemaHelpers.html#all_created?/1"},{"type":"function","title":"Example - EctoShorts.SchemaHelpers.all_created?/1","doc":"iex> SchemaHelpers.all_created?([%{id: 2}, %{\"id\" => 5}])\n  true\n  iex> SchemaHelpers.all_created?([%{\"id\" => 2}, %{item: 3}])\n  false","ref":"EctoShorts.SchemaHelpers.html#all_created?/1-example"},{"type":"function","title":"EctoShorts.SchemaHelpers.all_schemas?/1","doc":"Determine if all items in list are a schema","ref":"EctoShorts.SchemaHelpers.html#all_schemas?/1"},{"type":"function","title":"Example - EctoShorts.SchemaHelpers.all_schemas?/1","doc":"iex> SchemaHelpers.all_schemas([%{some_map: 1}, create_schema()])\n  false\n  iex> SchemaHelpers.all_schemas([create_schema(), create_schema()])\n  true","ref":"EctoShorts.SchemaHelpers.html#all_schemas?/1-example"},{"type":"function","title":"EctoShorts.SchemaHelpers.any_created?/1","doc":"Determine if any items in list has been created or not","ref":"EctoShorts.SchemaHelpers.html#any_created?/1"},{"type":"function","title":"Example - EctoShorts.SchemaHelpers.any_created?/1","doc":"iex> SchemaHelpers.any_created?([%{id: 2}, %{\"id\" => 5}])\n  true\n  iex> SchemaHelpers.any_created?([%{\"id\" => 2}, %{item: 3}])\n  true\n  iex> SchemaHelpers.any_created?([%{test: 3}, %{item: 3}])\n  false","ref":"EctoShorts.SchemaHelpers.html#any_created?/1-example"},{"type":"function","title":"EctoShorts.SchemaHelpers.created?/1","doc":"Determine if item has been created or not","ref":"EctoShorts.SchemaHelpers.html#created?/1"},{"type":"function","title":"Example - EctoShorts.SchemaHelpers.created?/1","doc":"iex> SchemaHelpers.created?(%{id: 2})\n  true\n  iex> SchemaHelpers.created?(%{\"id\" => 2})\n  true\n  iex> SchemaHelpers.created?(%{item: 3})\n  false","ref":"EctoShorts.SchemaHelpers.html#created?/1-example"},{"type":"function","title":"EctoShorts.SchemaHelpers.has_schemas?/1","doc":"Determine if any items in list are a schema","ref":"EctoShorts.SchemaHelpers.html#has_schemas?/1"},{"type":"function","title":"Example - EctoShorts.SchemaHelpers.has_schemas?/1","doc":"iex> SchemaHelpers.has_schemas?([%{some_map: 1}, create_schema()])\n  true\n  iex> SchemaHelpers.has_schemas?([%{some_map: 1}])\n  false","ref":"EctoShorts.SchemaHelpers.html#has_schemas?/1-example"},{"type":"function","title":"EctoShorts.SchemaHelpers.schema?/1","doc":"Determine if item passed in is a Ecto Schema","ref":"EctoShorts.SchemaHelpers.html#schema?/1"},{"type":"function","title":"Example - EctoShorts.SchemaHelpers.schema?/1","doc":"iex> SchemaHelpers.schema?(create_schema())\n  true\n  iex> SchemaHelpers.schema?(%{some_map: 1})\n  false\n  iex> SchemaHelpers.schema?([create_schema()])\n  false","ref":"EctoShorts.SchemaHelpers.html#schema?/1-example"},{"type":"behaviour","title":"EctoShorts.Actions.Error","doc":"This module generates errors from actions it can be\noverridden by config by setting error module\n\nErrors from Actions: [:not_found, :bad_request, :internal_server_error]","ref":"EctoShorts.Actions.Error.html"},{"type":"function","title":"EctoShorts.Actions.Error.call/3","doc":"","ref":"EctoShorts.Actions.Error.html#call/3"},{"type":"function","title":"EctoShorts.Actions.Error.create_error/3","doc":"","ref":"EctoShorts.Actions.Error.html#create_error/3"},{"type":"callback","title":"EctoShorts.Actions.Error.create_error/3","doc":"","ref":"EctoShorts.Actions.Error.html#c:create_error/3"},{"type":"function","title":"EctoShorts.Actions.Error.error_module/0","doc":"","ref":"EctoShorts.Actions.Error.html#error_module/0"},{"type":"type","title":"EctoShorts.Actions.Error.t/0","doc":"","ref":"EctoShorts.Actions.Error.html#t:t/0"},{"type":"behaviour","title":"EctoShorts.QueryBuilder","doc":"Behaviour for query building from filter tuples","ref":"EctoShorts.QueryBuilder.html"},{"type":"callback","title":"EctoShorts.QueryBuilder.create_schema_filter/2","doc":"Adds to accumulator query with filter_type and value","ref":"EctoShorts.QueryBuilder.html#c:create_schema_filter/2"},{"type":"function","title":"EctoShorts.QueryBuilder.create_schema_filter/3","doc":"","ref":"EctoShorts.QueryBuilder.html#create_schema_filter/3"},{"type":"function","title":"EctoShorts.QueryBuilder.query_schema/1","doc":"Pulls the schema from a query","ref":"EctoShorts.QueryBuilder.html#query_schema/1"},{"type":"type","title":"EctoShorts.QueryBuilder.accumulator_query/0","doc":"","ref":"EctoShorts.QueryBuilder.html#t:accumulator_query/0"},{"type":"type","title":"EctoShorts.QueryBuilder.filter_tuple/0","doc":"","ref":"EctoShorts.QueryBuilder.html#t:filter_tuple/0"},{"type":"module","title":"EctoShorts.QueryBuilder.Common","doc":"This module contains query building parts for common things such\nas preload, start/end date and others","ref":"EctoShorts.QueryBuilder.Common.html"},{"type":"function","title":"EctoShorts.QueryBuilder.Common.filters/0","doc":"","ref":"EctoShorts.QueryBuilder.Common.html#filters/0"},{"type":"module","title":"EctoShorts.QueryBuilder.Schema","doc":"This module contains query building parts for schemas themselves,\nwhen passed a query it can pull the schema from it and attempt\nto filter on any natural field","ref":"EctoShorts.QueryBuilder.Schema.html"},{"type":"function","title":"EctoShorts.QueryBuilder.Schema.create_schema_filter/3","doc":"","ref":"EctoShorts.QueryBuilder.Schema.html#create_schema_filter/3"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}